/*
 * The OpenSAML License, Version 1.
 * Copyright (c) 2002
 * University Corporation for Advanced Internet Development, Inc.
 * All rights reserved
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution, if any, must include
 * the following acknowledgment: "This product includes software developed by
 * the University Corporation for Advanced Internet Development
 * <http://www.ucaid.edu>Internet2 Project. Alternately, this acknowledegement
 * may appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * Neither the name of OpenSAML nor the names of its contributors, nor
 * Internet2, nor the University Corporation for Advanced Internet Development,
 * Inc., nor UCAID may be used to endorse or promote products derived from this
 * software without specific prior written permission. For written permission,
 * please contact opensaml@opensaml.org
 *
 * Products derived from this software may not be called OpenSAML, Internet2,
 * UCAID, or the University Corporation for Advanced Internet Development, nor
 * may OpenSAML appear in their name, without prior written permission of the
 * University Corporation for Advanced Internet Development.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND WITH ALL FAULTS. ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE DISCLAIMED AND THE ENTIRE RISK
 * OF SATISFACTORY QUALITY, PERFORMANCE, ACCURACY, AND EFFORT IS WITH LICENSEE.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER, CONTRIBUTORS OR THE UNIVERSITY
 * CORPORATION FOR ADVANCED INTERNET DEVELOPMENT, INC. BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
-----------------------------------------
file	:	SAMLAuthorizationDecisionQuery.cpp
project	:	Policy Server
date	:	
author	:	andy
rights	:	(c) Parthenon Computing
notes	:	 extension to openSAML C++ library (based on Java class)
--------------------------------------------
*/

#include "internal.h"

using namespace saml;
using namespace std;

SAMLAuthorizationDecisionQuery::SAMLAuthorizationDecisionQuery(SAMLSubject* subject, const XMLCh* resource, const Iterator<SAMLAction*>& actions, SAMLEvidence* evidence)
  : SAMLSubjectQuery(subject), m_resource(NULL), m_evidence(NULL)
{
    RTTI(SAMLAuthorizationDecisionQuery);
    if(!resource || *resource==chNull)
        throw MalformedException(SAMLException::REQUESTER,"SAMLAuthorizationDecisionQuery() requires resource");
    if(actions.size() == 0)
        throw MalformedException(SAMLException::REQUESTER,"SAMLAuthorizationDecisionQuery() requires at least one action");

    m_subject = subject;
    m_resource = XMLString::replicate(resource);
    while(actions.hasNext())
        m_actions.push_back(actions.next());

    if(evidence)
        m_evidence = evidence;
}

SAMLAuthorizationDecisionQuery::SAMLAuthorizationDecisionQuery(DOMElement* e)
    : SAMLSubjectQuery(e), m_resource(NULL), m_evidence(NULL)
{
    RTTI(SAMLAuthorizationDecisionQuery);
    fromDOM(e);
}

SAMLAuthorizationDecisionQuery::SAMLAuthorizationDecisionQuery(std::istream& in)
    : SAMLSubjectQuery(in), m_resource(NULL), m_evidence(NULL)
{
    RTTI(SAMLAuthorizationDecisionQuery);
    fromDOM(m_document->getDocumentElement());
}

SAMLAuthorizationDecisionQuery::~SAMLAuthorizationDecisionQuery()
{
    if(m_bOwnStrings)
        XMLString::release(const_cast<XMLCh**>(&m_resource));
    for (vector<SAMLAction*>::const_iterator i=m_actions.begin(); i!=m_actions.end(); i++)
        delete (*i);
    delete m_evidence;
}

void SAMLAuthorizationDecisionQuery::checkValidity() const
{
    SAMLSubjectQuery::checkValidity();
    if (!m_resource || *m_resource==chNull || m_actions.size()==0)
        throw MalformedException(SAMLException::REQUESTER, "SAMLAuthorizationDecisionQuery requires a resource and at least one action");
}

SAMLObject* SAMLAuthorizationDecisionQuery::clone() const
{
    return new SAMLAuthorizationDecisionQuery(
        static_cast<SAMLSubject*>(m_subject->clone()), 
        m_resource, 
        getActions().clone(), 
        m_evidence ? static_cast<SAMLEvidence*>(m_evidence->clone()) : NULL
        );
}

void SAMLAuthorizationDecisionQuery::fromDOM(DOMElement* e)
{
    NDC ndc("fromDOM");

    if (SAMLConfig::getConfig().strict_dom_checking) {
        if (XMLString::compareString(XML::SAMLP_NS,e->getNamespaceURI()))
            throw MalformedException(SAMLException::REQUESTER,"SAMLAuthorizationDecisionQuery::fromDOM() root element isn't in samlp namespace");
        if (XMLString::compareString(L(AuthorizationDecisionQuery),e->getLocalName())) {
            auto_ptr<saml::QName> type(saml::QName::getQNameAttribute(e,XML::XSI_NS,L(type)));
            if ((XMLString::compareString(L(Query),e->getLocalName()) && XMLString::compareString(L(SubjectQuery),e->getLocalName())) ||
                !type.get() || XMLString::compareString(XML::SAMLP_NS,type->getNamespaceURI()) ||
                XMLString::compareString(L(AuthorizationDecisionQueryType),type->getLocalName()))
                throw MalformedException(SAMLException::REQUESTER,"SAMLAuthorizationDecisionQuery::fromDOM() missing samlp:AuthorizationDecisionQuery element at root");
        }
    }

    m_resource = e->getAttributeNS(NULL,L(Resource));

    DOMElement* n=XML::getFirstChildElement(e);
    while(n) {
        // the top-level children are either actions (mandatory, unbounded) or evidence (optional)
        if (XML::isElementNamed(n,XML::SAML_NS,L(Action))) {
            m_actions.push_back(new SAMLAction(n));
        }
        else if (XML::isElementNamed(n,XML::SAML_NS,L(Evidence))) {
            m_evidence = new SAMLEvidence(n);
        }
        n=XML::getNextSiblingElement(n);
    } 

    checkValidity();
}

DOMNode* SAMLAuthorizationDecisionQuery::toDOM(DOMDocument* doc, bool xmlns) const
{
    SAMLObject::toDOM(doc, xmlns);
    if(m_root) {
        if(xmlns)
            static_cast<DOMElement*>(m_root)->setAttributeNS(XML::XMLNS_NS,L(xmlns),XML::SAMLP_NS);
        return m_root;
    }
    if (!doc)
        doc=m_document;

    // Construct an AuthorizationDecisionQuery
    DOMElement* q=doc->createElementNS(XML::SAMLP_NS,L(AuthorizationDecisionQuery));
    if (xmlns)
        q->setAttributeNS(XML::XMLNS_NS,L(xmlns),XML::SAMLP_NS);
    q->setAttributeNS(NULL,L(Resource),m_resource);

    q->appendChild(m_subject->toDOM(doc,true));

    for (vector<SAMLAction*>::const_iterator i=m_actions.begin(); i!=m_actions.end(); i++)
        q->appendChild((*i)->toDOM(doc,true));

    if(m_evidence)
        q->appendChild(m_evidence->toDOM(doc,true));

    return m_root=q;
}

