/*
 * The OpenSAML License, Version 1.
 * Copyright (c) 2002
 * University Corporation for Advanced Internet Development, Inc.
 * All rights reserved
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution, if any, must include
 * the following acknowledgment: "This product includes software developed by
 * the University Corporation for Advanced Internet Development
 * <http://www.ucaid.edu>Internet2 Project. Alternately, this acknowledegement
 * may appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * Neither the name of OpenSAML nor the names of its contributors, nor
 * Internet2, nor the University Corporation for Advanced Internet Development,
 * Inc., nor UCAID may be used to endorse or promote products derived from this
 * software without specific prior written permission. For written permission,
 * please contact opensaml@opensaml.org
 *
 * Products derived from this software may not be called OpenSAML, Internet2,
 * UCAID, or the University Corporation for Advanced Internet Development, nor
 * may OpenSAML appear in their name, without prior written permission of the
 * University Corporation for Advanced Internet Development.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND WITH ALL FAULTS. ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE DISCLAIMED AND THE ENTIRE RISK
 * OF SATISFACTORY QUALITY, PERFORMANCE, ACCURACY, AND EFFORT IS WITH LICENSEE.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER, CONTRIBUTORS OR THE UNIVERSITY
 * CORPORATION FOR ADVANCED INTERNET DEVELOPMENT, INC. BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
-----------------------------------------
file	:	SAMLAuthorizationDecisionStatement.cpp
project	:	Policy Server
date	:	
author	:	andy
rights	:	(c) Parthenon Computing
notes	:	 extension to openSAML C++ library (based on Java class)
--------------------------------------------
*/

#include "internal.h"

using namespace saml;
using namespace std;

SAMLAuthorizationDecisionStatement::SAMLAuthorizationDecisionStatement(
    SAMLSubject* subject, const XMLCh* resource, const XMLCh* decision, const Iterator<SAMLAction*>& actions, SAMLEvidence* evidence
    ) : SAMLSubjectStatement(subject), m_resource(NULL), m_decision(NULL), m_evidence(NULL)
{
    RTTI(SAMLAuthorizationDecisionStatement);

     if(!resource || *resource==chNull)
        throw SAMLException(SAMLException::REQUESTER,"SAMLAuthorizationDecisionStatement() requires resource");
    if(!decision || *decision==chNull)
        throw SAMLException(SAMLException::REQUESTER,"SAMLAuthorizationDecisionStatement() requires decision");
    if(actions.size() == 0)
        throw SAMLException(SAMLException::REQUESTER,"SAMLAuthorizationDecisionStatement() requires at least one action");

    m_resource = XMLString::replicate(resource);
    m_decision = XMLString::replicate(decision);
    while(actions.hasNext())
        m_actions.push_back(actions.next());

    if(evidence)
        m_evidence = evidence;
}

SAMLAuthorizationDecisionStatement::SAMLAuthorizationDecisionStatement(DOMElement* e)
  : SAMLSubjectStatement(e), m_resource(NULL), m_decision(NULL), m_evidence(NULL)
{
    RTTI(SAMLAuthorizationDecisionStatement);
    fromDOM(e);
}

SAMLAuthorizationDecisionStatement::SAMLAuthorizationDecisionStatement(std::istream& in)
  : SAMLSubjectStatement(in), m_resource(NULL), m_decision(NULL), m_evidence(NULL)
{
    RTTI(SAMLAuthorizationDecisionStatement);
    fromDOM(m_document->getDocumentElement());
}

SAMLAuthorizationDecisionStatement::~SAMLAuthorizationDecisionStatement()
{
    if(m_bOwnStrings) {
        XMLString::release(const_cast<XMLCh**>(&m_resource));
        XMLString::release(const_cast<XMLCh**>(&m_decision));
    }
    for (vector<SAMLAction*>::const_iterator i=m_actions.begin(); i!=m_actions.end(); i++)
        delete (*i);
    delete m_evidence;
}

void SAMLAuthorizationDecisionStatement::checkValidity() const
{
    SAMLSubjectStatement::checkValidity();
    if (!m_resource || *m_resource==chNull || !m_decision || *m_decision==chNull ||m_actions.size() ==0)
        throw MalformedException(SAMLException::RESPONDER, "SAMLAuthorizationDecisionStatement invalid, requires a resource, a decision and at least one action");
}

SAMLObject* SAMLAuthorizationDecisionStatement::clone() const
{
    return new SAMLAuthorizationDecisionStatement(
        static_cast<SAMLSubject*>(m_subject->clone()),
        m_resource, 
        m_decision,
        getActions().clone(), 
        m_evidence ? static_cast<SAMLEvidence*>(m_evidence->clone()) : NULL
        );
}

void SAMLAuthorizationDecisionStatement::fromDOM(DOMElement* e)
{
    NDC ndc("fromDOM");

    if (SAMLConfig::getConfig().strict_dom_checking) {
    if (XMLString::compareString(XML::SAML_NS,e->getNamespaceURI()))
        throw MalformedException(SAMLException::RESPONDER,"SAMLAuthorizationDecisionStatement::fromDOM() missing saml namespace on root element");

    if (XMLString::compareString(L(AuthorizationDecisionStatement),e->getLocalName())) {
        auto_ptr<saml::QName> type(saml::QName::getQNameAttribute(e,XML::XSI_NS,L(type)));
        if ((XMLString::compareString(L(Statement),e->getLocalName()) && XMLString::compareString(L(SubjectStatement),e->getLocalName())) ||
            !type.get() || XMLString::compareString(XML::SAML_NS,type->getNamespaceURI()) ||
            XMLString::compareString(L(AuthorizationDecisionStatementType),type->getLocalName()))
            throw MalformedException(SAMLException::RESPONDER, "SAMLAuthorizationDecisionStatement::fromDOM() requires saml:AuthorizationDecisionStatement at root");
        }
    }

    // attributes: resource, decision
    m_resource = e->getAttributeNS(NULL,L(Resource));
    m_decision = e->getAttributeNS(NULL,L(Decision));

    // the remaining top-level children (subject has been deserialized in the base
    // class) are either actions or evidence
    DOMElement* n=XML::getFirstChildElement(e);
    while(n) {
        if (XML::isElementNamed(n,XML::SAML_NS,L(Action))) {
            m_actions.push_back(new SAMLAction(n));
        }
        else if (XML::isElementNamed(n,XML::SAML_NS,L(Evidence))) {
            m_evidence = new SAMLEvidence(n);
        }
        n=XML::getNextSiblingElement(n);
    } 

    checkValidity();
}

DOMNode* SAMLAuthorizationDecisionStatement::toDOM(DOMDocument* doc, bool xmlns) const
{
    SAMLObject::toDOM(doc, xmlns);
    if(m_root) {
        if(xmlns)
            static_cast<DOMElement*>(m_root)->setAttributeNS(XML::XMLNS_NS,L(xmlns),XML::SAML_NS);
        return m_root;
    }
    if (!doc)
        doc=m_document;

    // Construct an AuthorizationDecisionStatement
    DOMElement* s=doc->createElementNS(XML::SAML_NS,L(AuthorizationDecisionStatement));
    if (xmlns)
        s->setAttributeNS(XML::XMLNS_NS,L(xmlns),XML::SAML_NS);

    s->setAttributeNS(NULL,L(Resource),m_resource);
    s->setAttributeNS(NULL,L(Decision),m_decision);

    s->appendChild(m_subject->toDOM(doc,false));

    for (vector<SAMLAction*>::const_iterator i=m_actions.begin(); i!=m_actions.end(); i++)
        s->appendChild((*i)->toDOM(doc,false));

    if(m_evidence)
        s->appendChild(m_evidence->toDOM(doc,false));

    return m_root=s;
}

