/*
 * The OpenSAML License, Version 1.
 * Copyright (c) 2002
 * University Corporation for Advanced Internet Development, Inc.
 * All rights reserved
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution, if any, must include
 * the following acknowledgment: "This product includes software developed by
 * the University Corporation for Advanced Internet Development
 * <http://www.ucaid.edu>Internet2 Project. Alternately, this acknowledegement
 * may appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * Neither the name of OpenSAML nor the names of its contributors, nor
 * Internet2, nor the University Corporation for Advanced Internet Development,
 * Inc., nor UCAID may be used to endorse or promote products derived from this
 * software without specific prior written permission. For written permission,
 * please contact opensaml@opensaml.org
 *
 * Products derived from this software may not be called OpenSAML, Internet2,
 * UCAID, or the University Corporation for Advanced Internet Development, nor
 * may OpenSAML appear in their name, without prior written permission of the
 * University Corporation for Advanced Internet Development.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND WITH ALL FAULTS. ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE DISCLAIMED AND THE ENTIRE RISK
 * OF SATISFACTORY QUALITY, PERFORMANCE, ACCURACY, AND EFFORT IS WITH LICENSEE.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER, CONTRIBUTORS OR THE UNIVERSITY
 * CORPORATION FOR ADVANCED INTERNET DEVELOPMENT, INC. BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
-----------------------------------------
file	:	SAMLEvidence.cpp
project	:	Policy Server
date	:	
author	:	andy
rights	:	(c) Parthenon Computing
notes	:	 extension to openSAML C++ library (based on Java class)
--------------------------------------------
*/

#include "internal.h"

using namespace saml;
using namespace std;

SAMLEvidence::SAMLEvidence(const Iterator<SAMLAssertion*>& assertions, const Iterator<const XMLCh*>& assertionIDRefs)
{
    construct(assertions, assertionIDRefs);
}

SAMLEvidence::SAMLEvidence(const Iterator<const XMLCh*>& assertionIDRefs, const Iterator<SAMLAssertion*>& assertions)
{
    construct(assertions, assertionIDRefs);
}

void SAMLEvidence::construct(const Iterator<SAMLAssertion*>& assertions, const Iterator<const XMLCh*>& assertionIDRefs)
{
    RTTI(SAMLEvidence);

    if(assertions.size() == 0 && assertionIDRefs.size() == 0)
        throw MalformedException("SAMLEvidence() requires at least one of either an assertion or an assertion ID reference");

    while(assertions.hasNext())
        m_assertions.push_back(assertions.next());

    while(assertionIDRefs.hasNext())
        m_assertionIDRefs.push_back(XMLString::replicate(assertionIDRefs.next()));
}

SAMLEvidence::SAMLEvidence(DOMElement* e)
{
    RTTI(SAMLEvidence);
    fromDOM(e);
}

SAMLEvidence::SAMLEvidence(std::istream& in) : SAMLObject(in)
{
    RTTI(SAMLEvidence);
    fromDOM(m_document->getDocumentElement());
}

SAMLEvidence::~SAMLEvidence()
{
    for (vector<SAMLAssertion*>::const_iterator i=m_assertions.begin(); i!=m_assertions.end(); i++)
        delete (*i);

    if(m_bOwnStrings) {
        for (vector<const XMLCh*>::const_iterator i=m_assertionIDRefs.begin(); i!=m_assertionIDRefs.end(); i++) {
            XMLCh* p = const_cast<XMLCh*>(*i);
            XMLString::release(&p);
        }
    }
}

void SAMLEvidence::checkValidity() const
{
    if(m_assertions.size() == 0 && m_assertionIDRefs.size() == 0)
        throw MalformedException("SAMLEvidence() requires at least one of either an assertion or an assertion ID reference");
}

SAMLObject* SAMLEvidence::clone() const
{
    return new SAMLEvidence(getAssertions().clone(), m_assertionIDRefs);
}

void SAMLEvidence::fromDOM(DOMElement *e)
{
    SAMLObject::fromDOM(e);

    if (SAMLConfig::getConfig().strict_dom_checking &&
        (XMLString::compareString(XML::SAML_NS, e->getNamespaceURI()) || XMLString::compareString(L(Evidence), e->getLocalName())))
        throw MalformedException("SAMLEvidence() requires saml:Evidence at root");

    m_root = e;
    m_bOwnStrings = false;

    // iterate over assertions
    DOMNodeList *nlist = (static_cast<DOMElement*>(e))->getElementsByTagNameNS(XML::SAML_NS, L(Assertion));
    for (int i=0; i<nlist->getLength(); i++)
        m_assertions.push_back(new SAMLAssertion((DOMElement*)nlist->item(i)));

    // iterate over assertion ID references
    nlist = (static_cast<DOMElement*>(e))->getElementsByTagNameNS(XML::SAML_NS, L(AssertionIDReference));
    for (int j=0; j<nlist->getLength(); j++)
        m_assertionIDRefs.push_back(nlist->item(j)->getFirstChild()->getNodeValue());

    checkValidity();
}

DOMNode* SAMLEvidence::toDOM(DOMDocument* doc, bool xmlns) const
{
    SAMLObject::toDOM(doc, xmlns);
    if (m_root) {
        if (xmlns)
            static_cast<DOMElement*>(m_root)->setAttributeNS(XML::XMLNS_NS,L(xmlns),XML::SAML_NS);
        return m_root;
    }

    if (!doc)
        doc=m_document;

    // Construct an Evidence
    DOMElement* e=doc->createElementNS(XML::SAML_NS, L(Evidence));
    if (xmlns)
        e->setAttributeNS(XML::XMLNS_NS, L(xmlns), XML::SAML_NS);

    for (vector<SAMLAssertion*>::const_iterator i=m_assertions.begin(); i!=m_assertions.end(); i++)
        e->appendChild((*i)->toDOM(doc,false));

    for (vector<const XMLCh*>::const_iterator j=m_assertionIDRefs.begin(); j!=m_assertionIDRefs.end(); j++)
        e->appendChild(doc->createElementNS(XML::SAML_NS,L(AssertionIDReference)))->appendChild(doc->createTextNode(*j));

    return m_root=e;
}
