/*
 * The OpenSAML License, Version 1.
 * Copyright (c) 2002
 * University Corporation for Advanced Internet Development, Inc.
 * All rights reserved
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution, if any, must include
 * the following acknowledgment: "This product includes software developed by
 * the University Corporation for Advanced Internet Development
 * <http://www.ucaid.edu>Internet2 Project. Alternately, this acknowledegement
 * may appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * Neither the name of OpenSAML nor the names of its contributors, nor
 * Internet2, nor the University Corporation for Advanced Internet Development,
 * Inc., nor UCAID may be used to endorse or promote products derived from this
 * software without specific prior written permission. For written permission,
 * please contact opensaml@opensaml.org
 *
 * Products derived from this software may not be called OpenSAML, Internet2,
 * UCAID, or the University Corporation for Advanced Internet Development, nor
 * may OpenSAML appear in their name, without prior written permission of the
 * University Corporation for Advanced Internet Development.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND WITH ALL FAULTS. ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE DISCLAIMED AND THE ENTIRE RISK
 * OF SATISFACTORY QUALITY, PERFORMANCE, ACCURACY, AND EFFORT IS WITH LICENSEE.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER, CONTRIBUTORS OR THE UNIVERSITY
 * CORPORATION FOR ADVANCED INTERNET DEVELOPMENT, INC. BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
-----------------------------------------
file	:	tester.cpp
project	:	Policy Server
date	:	
author	:	andy
notes	:	test harness for the openSAML extensions
--------------------------------------------
*/

#include "saml/saml.h"

#include <fstream>
#include <sstream>

using namespace std;
using namespace saml;
XERCES_CPP_NAMESPACE_USE

// choice of where toDOM() output goes
enum DOM_DUMP
{
  NONE,
  SCREEN,
  LOCALFILE
};

// prototypes here so that main() comes first
void usage(char* name);
void welcome();
void serialize(SAMLObject *obj, DOM_DUMP dumpTo);

void testSAMLAction();
void testSAMLEvidence();
void testSAMLAuthorizationDecisionQuery();
void testSAMLAuthorizationDecisionStatement();

void testSAMLAttributeQuery();

// configuration data
static DOM_DUMP    gSerializeTo                      = NONE;
static const char* gExamplesDir                      = "../data";
static const char* gSchemaDir                        = "../schemas";
static bool        gSkipDeserializationFromInputStream = false;
static bool        gErrors                           = false;
static string      testClass;

int main(int argc, char **argv)
{
    // process arguments
	for(int i=1; i<argc; i++) {
        if(argv[i][0]=='-') {
            if(!strncmp(argv[i], "-h", 2) || !strncmp(argv[i], "-?", 2)) {
                usage(argv[0]);
                welcome();
                return 1;
            }
            else if(!strncmp(argv[i], "-sdir", 5)) {
                gSchemaDir = argv[++i];
            }
            else if(!strncmp(argv[i], "-edir", 5)) {
                char* arg = argv[++i];
    
                // ensure a trailing slash
                size_t len = strlen(arg);
                if(arg[len-1] != '/')
                    strcat(arg, "/");
    
                gExamplesDir = arg;
            }
            else if(!strncmp(argv[i], "-dump", 5)) {
                const char* const arg = argv[++i];
                if(!strcmp(arg, "screen"))
                    gSerializeTo = SCREEN;
                else if(!strcmp(arg, "file")) 
                    gSerializeTo = LOCALFILE;
                else
                    gSerializeTo = NONE;
            }
            else if(!strncmp(argv[i], "-skipis", 7)) {
                gSkipDeserializationFromInputStream = true;
            }
            else {
                const char* const arg = argv[++i];
                cerr << "\nUnknown argument '" << arg << "'\n";
        				usage(argv[0]);
                return 1;
            }
        }
    }


    // explain what is about to happen
    welcome();
    cerr << endl;
    cerr << "Schema directory is \"" << gSchemaDir << "\"\n";
    cerr << "Node serialization will ";
    switch(gSerializeTo) {
        case NONE:
            cerr << "not be written anywhere";
            break;
        case SCREEN:
            cerr << "be printed to stdout";
            break;
        case LOCALFILE:
            cerr << "be written to \"<element-name>.xml\"";
            break;
    }
    cerr << endl;
    if(gSkipDeserializationFromInputStream)
        cerr << "Skipping deserialization from input streams\n";
    else
        cerr << "Deserializing from files in \"" << gExamplesDir << "\"\n";

    cerr << endl;

    // initialize SAML, specifying the schema directory
    SAMLConfig& conf=SAMLConfig::getConfig();
    conf.schema_dir = gSchemaDir;
    if (!conf.init()) {
        cerr << "Error initializing SAML runtime" << endl;
        return -1;
    }

    // bring on the tests...
    try {
        testSAMLAction();
        testSAMLEvidence();
        testSAMLAuthorizationDecisionQuery();
        testSAMLAuthorizationDecisionStatement();
        testSAMLAttributeQuery();
    }
    catch(const XMLException &e) {
        cerr << endl << "FAILED: " << testClass << ":" << endl;
        auto_ptr_char what(e.getMessage());
        cerr << "Xerces-C exception caught: " << what.get() << endl;
        gErrors=true;
    }
    catch(const SAMLException &e) {
        cerr << endl << "FAILED: " << testClass << ":" << endl;
        cerr << "SAML exception caught: " << e.what() << endl;
        gErrors=true;
    }
    catch(...) {
        cerr << endl << "FAILED: " << testClass << ":" << endl;
        cerr << "Unknown exception caught" << endl;
        gErrors=true;
    }

    // house keeping
    conf.term();
    if (gErrors) {
        cerr << "\nCompleted the test run with errors.\n\n";
        return -1;
    }
            
    cerr << "\nCompleted the test run without errors.\n\n";
    return 0;
}

void usage(char *name)
{
    cerr << "Test harness for openSAML C++ extensions\n";
    cerr << name << " [options]\n\n";
    cerr << "Options:\n";
    cerr << "  -dump xxx    Serialize to [file | screen | none*]\n";
    cerr << "  -sdir xxx    Schema directory (required if deserializing input streams (.)\n";
    cerr << "  -edir xxx    Directory containing files for deserialization (.)\n";
    cerr << "  -skipis      Skip the deserialization from input streams\n";
    cerr << "  -h           Display this usage message\n";

    cerr << "\nWhen running for the first time, set the '-skipis' flag and serialize\nto file. "
        << "Copy the files to the directory that will be identified by the\n'-edir' flag in future runs.\n";
}

void welcome()
{
    cerr << "\n";
    cerr << "Each class undergoes the following tests:\n";
    cerr << "  - basic construction\n";
    cerr << "  - serialize from DOM\n";
    cerr << "  - getter methods\n";
    cerr << "  - cloning\n";
    cerr << "  - deserialization\n";
    cerr << "  - serialize from input stream (optional) \n\n";
}

// helper method - serialize object and optionally dump output
void serialize(SAMLObject* obj, DOM_DUMP dumpTo)
{
    cerr << "\t\tSerialization...";

    if(dumpTo == NONE) {
        DOMNode* n = obj->toDOM();
        cerr << "OK" << endl;
    }
    else {
        if (dumpTo == LOCALFILE) {
            auto_ptr_char ele(obj->toDOM()->getNodeName());
            string file = string("SAML") + ele.get() + ".xml";
            ofstream ostr(file.c_str());
            ostr << *obj;
            cerr << "OK" << endl;
        }
        else {
            cerr << endl << ">>>>>" << endl;
            cout << *obj;
            cerr << endl << "<<<<<" << endl << "\t\tOK" << endl;
        }
    }
}

///////////////////////////////////////////////////////////////////////////
// SAMLAction
void testSAMLAction()
{
    testClass = "SAMLAction";
    cerr << endl << testClass << endl;

    // 1 - Action with no namespace
    {
        cerr << "\tAction with no namespace" << endl;

        // CTOR: action with no namespace specified
        auto_ptr_XMLCh strAction("read");
        SAMLAction *action = new SAMLAction(strAction.get());

        // validity
        cerr << "\t\tValid object...";
        action->checkValidity();
        cerr << "OK" << endl;

        // Build from DOMElement
        cerr << "\t\tBuild from DOMElement...";
        SAMLAction *actionFromDOM = new SAMLAction(static_cast<DOMElement*>(action->toDOM()));
        delete actionFromDOM;
        cerr << "OK" << endl;

        // getter methods
        cerr << "\t\tGetter methods...";
        const XMLCh* d = action->getData();
        if(!XMLString::equals(strAction.get(), d)) {
            throw SAMLException("getData() returned incorrect value");
        }
        const XMLCh *ns = action->getNamespace();
        if(!XMLString::equals(SAMLAction::SAML_ACTION_NAMESPACE_RWEDC_NEG, ns)) {
            throw SAMLException("getNamespace() returned incorrect value");
        }
        cerr << "OK" << endl;

        // clone
        cerr << "\t\tClone...";
        auto_ptr<SAMLAction> clonedAction(static_cast<SAMLAction*>(action->clone()));
        delete action;

        clonedAction->checkValidity();
        if(!XMLString::equals(strAction.get(), clonedAction->getData())) {
            throw SAMLException("getData() on cloned object returned incorrect value");
        }
        cerr << "OK" << endl;

        // serialization
        serialize(clonedAction.get(), gSerializeTo);

        // serialize from input stream
        if(!gSkipDeserializationFromInputStream) {
            cerr << "\t\tDeserialize from an input stream...";
            string file = gExamplesDir + testClass + ".xml";
            ifstream s(file.c_str());
            auto_ptr<SAMLAction> actionFromStream(new SAMLAction(s));
            actionFromStream->checkValidity();
            cerr << "OK" << endl;
        }

        cerr << "\tOK" << endl;
    }

    // 2 - Action with a namespace
    {
        cerr << "\tAction with a namespace..." << endl;

        // CTOR: action with a namespace
        auto_ptr_XMLCh strAction("read");
        SAMLAction* action = new SAMLAction(strAction.get(), SAMLAction::SAML_ACTION_NAMESPACE_RWEDC);

        cerr << "\t\tValid object...";
        action->checkValidity();
        cerr << "OK" << endl;

        // Build from DOMElement
        cerr << "\t\tBuild from DOMElement...";
        SAMLAction* actionFromDOM = new SAMLAction(static_cast<DOMElement*>(action->toDOM()));
        delete actionFromDOM;
        cerr << "OK" << endl;

        // getter methods
        cerr << "\t\tGetter methods...";
        if(!XMLString::equals(strAction.get(), action->getData())) {
            throw SAMLException("getData() returned incorrect value");
        }
        if(!XMLString::equals(action->getNamespace(),SAMLAction::SAML_ACTION_NAMESPACE_RWEDC)) {
            throw SAMLException("getNamespace() returned incorrect value");
        }
        cerr << "OK" << endl;

        // clone
        cerr << "\t\tClone...";
        auto_ptr<SAMLAction> clonedAction(static_cast<SAMLAction*>(action->clone()));
        delete action;

        clonedAction->checkValidity();
        if(!XMLString::equals(clonedAction->getNamespace(),SAMLAction::SAML_ACTION_NAMESPACE_RWEDC)) {
            throw SAMLException("getNamespace() on cloned object returned incorrect value");
        }

        cerr << "OK" << endl;

        // serialization
        serialize(clonedAction.get(), gSerializeTo);

        // serialize from input stream
        if(!gSkipDeserializationFromInputStream) {
            cerr << "\t\tDeserialize from an input stream...";
            string file = gExamplesDir + testClass + ".xml";
            ifstream s(file.c_str());
            auto_ptr<SAMLAction> actionFromStream(new SAMLAction(s));
            actionFromStream->checkValidity();
            cerr << "OK" << endl;
        }

        cerr << "\tOK" << endl;
    }
}

///////////////////////////////////////////////////////////////////////////
// SAMLEvidence

SAMLAssertion *createAssertion(const XMLCh *issuer);

void testSAMLEvidence()
{
    testClass = "SAMLEvidence";
    cerr << endl << testClass << endl;

    // 1 - Evidence with one assertion and no assertion ID refs
    {
        cerr << "\tEvidence with one assertion" << endl;
        auto_ptr_XMLCh issuer("http://www.example.com");

        SAMLAssertion* assertion = createAssertion(issuer.get());
        SAMLAssertion* av[] = { assertion };
        SAMLEvidence* evidence = new SAMLEvidence(ArrayIterator<SAMLAssertion*>(av,1));

        cerr << "\t\tValid object...";
        evidence->checkValidity();
        cerr << "OK" << endl;

        // Build from DOMElement
        cerr << "\t\tBuild from DOMElement...";
        SAMLEvidence* evidenceFromDOM = new SAMLEvidence(static_cast<DOMElement*>(evidence->toDOM()));
        evidenceFromDOM->checkValidity();
        delete evidenceFromDOM;
        cerr << "OK" << endl;

        // getter methods
        cerr << "\t\tGetter methods...";
        if(evidence->getAssertions().size() != 1) {
            throw SAMLException("getAssertions() returned incorrect value");
        }
        for (Iterator<SAMLAssertion*> assertions=evidence->getAssertions(); assertions.hasNext();) {
            const SAMLAssertion *a = assertions.next();
            if(!XMLString::equals(issuer.get(), a->getIssuer()))
                throw SAMLException("unexpected issuer");
        }

        if(evidence->getAssertionIDRefs().size() != 0) {
            throw SAMLException("getAssertionIDRefs() returned incorrect value");
        }
        cerr << "OK" << endl;

        // clone
        cerr << "\t\tClone...";
        auto_ptr<SAMLEvidence> clonedEvidence(static_cast<SAMLEvidence*>(evidence->clone()));
        delete evidence;

        clonedEvidence->checkValidity();
        if (clonedEvidence->getAssertions().size() != 1) {
            throw SAMLException("getAssertions() on clone returned incorrect value");
        }
        if (clonedEvidence->getAssertionIDRefs().size() != 0) {
            throw SAMLException("getAssertionIDRefs() on clone returned incorrect value");
        }
        cerr << "OK" << endl;

        // serialization
        serialize(clonedEvidence.get(), gSerializeTo);

        // deserialize from input stream
        if(!gSkipDeserializationFromInputStream) {
            cerr << "\t\tDeserialize from an input stream...";
            string file = gExamplesDir + testClass + ".xml";
            ifstream s(file.c_str());
            auto_ptr<SAMLEvidence> evidenceFromStream(new SAMLEvidence(s));
            evidenceFromStream->checkValidity();
            cerr << "OK" << endl;
        }

        cerr << "\tOK" << endl;
    }

    // 2 - Evidence with no assertions and one assertion ID ref
    {
        cerr << "\tEvidence with one assertion ID reference" << endl;
        auto_ptr_XMLCh idref("ID666");
        const XMLCh* idrefv[] = { idref.get() };
        SAMLEvidence* evidence = new SAMLEvidence(ArrayIterator<const XMLCh*>(idrefv,1));

        cerr << "\t\tValid object...";
        evidence->checkValidity();
        cerr << "OK" << endl;

        // getter methods
        cerr << "\t\tGetter methods...";
        if (evidence->getAssertions().size() != 0) {
            throw SAMLException("getAssertions() returned incorrect value");
        }

        if (evidence->getAssertionIDRefs().size() != 1) {
            throw SAMLException("getAssertionIDRefs() returned incorrect value");
        }
        for (Iterator<const XMLCh*> ids=evidence->getAssertionIDRefs(); ids.hasNext();) {
            if(!XMLString::equals(idref.get(), ids.next()))
                throw SAMLException("unexpected assertion ref id");
        }
        cerr << "OK" << endl;

        // clone
        cerr << "\t\tClone...";
        auto_ptr<SAMLEvidence> clonedEvidence(static_cast<SAMLEvidence*>(evidence->clone()));
        delete evidence;

        clonedEvidence->checkValidity();
        if (clonedEvidence->getAssertions().size() != 0) {
            throw SAMLException("getAssertions() on clone returned incorrect value");
        }
        if (clonedEvidence->getAssertionIDRefs().size() != 1) {
            throw SAMLException("getAssertionIDRefs() on clone returned incorrect value");
        }
        cerr << "OK" << endl;

        // serialization
        serialize(clonedEvidence.get(), gSerializeTo);

        // deserialize from input stream
        if(!gSkipDeserializationFromInputStream) {
            cerr << "\t\tDeserialize from an input stream...";
            string file = gExamplesDir + testClass + ".xml";
            ifstream s(file.c_str());
            auto_ptr<SAMLEvidence> evidenceFromStream(new SAMLEvidence(s));
            evidenceFromStream->checkValidity();
            cerr << "OK" << endl;
        }

        cerr << "\tOK" << endl;
    }

    // 3 - Evidence with one assertion and one assertion ID ref
    {
        cerr << "\tEvidence with one assertion and one assertion id ref" << endl;
        auto_ptr_XMLCh issuer("http://www.example.com");
        SAMLAssertion* assertion = createAssertion(issuer.get());
        SAMLAssertion* av[] = { assertion };

        auto_ptr_XMLCh idref("ID666");
        const XMLCh* idrefv[] = { idref.get() };

        SAMLEvidence* evidence = new SAMLEvidence(ArrayIterator<SAMLAssertion*>(av,1), ArrayIterator<const XMLCh*>(idrefv,1));

        cerr << "\t\tValid object...";
        evidence->checkValidity();
        cerr << "OK" << endl;

        // serialize from DOMElement
        cerr << "\t\tSerialize from DOMElement...";
        SAMLEvidence* evidenceFromDOM = new SAMLEvidence(static_cast<DOMElement*>(evidence->toDOM()));
        evidenceFromDOM->checkValidity();
        delete evidenceFromDOM;
        cerr << "OK" << endl;

        // getter methods
        cerr << "\t\tGetter methods...";
        if (evidence->getAssertions().size() != 1) {
            throw SAMLException("getAssertions() returned incorrect value");
        }
        for (Iterator<SAMLAssertion*> assertions=evidence->getAssertions(); assertions.hasNext();) {
            const SAMLAssertion *a = assertions.next();
            if(!XMLString::equals(issuer.get(), a->getIssuer()))
                throw SAMLException("unexpected issuer");
        }

        if (evidence->getAssertionIDRefs().size() != 1) {
            throw SAMLException("getAssertionIDRefs() returned incorrect value");
        }
        for (Iterator<const XMLCh*> ids=evidence->getAssertionIDRefs(); ids.hasNext();) {
            if(!XMLString::equals(idref.get(), ids.next()))
                throw SAMLException("unexpected assertion ref id");
        }
        cerr << "OK" << endl;

        // clone
        cerr << "\t\tClone...";
        auto_ptr<SAMLEvidence> clonedEvidence(static_cast<SAMLEvidence*>(evidence->clone()));
        delete evidence;

        clonedEvidence->checkValidity();
        if (clonedEvidence->getAssertions().size() != 1) {
            throw SAMLException("getAssertions() on clone returned incorrect value");
        }
        if (clonedEvidence->getAssertionIDRefs().size() != 1) {
            throw SAMLException("getAssertionIDRefs() on clone returned incorrect value");
        }
        cerr << "OK" << endl;

        // serialization
        serialize(clonedEvidence.get(), gSerializeTo);

        // deserialize from input stream
        if(!gSkipDeserializationFromInputStream) {
            cerr << "\t\tDeserialize from an input stream...";
            string file = gExamplesDir + testClass + ".xml";
            ifstream s(file.c_str());
            auto_ptr<SAMLEvidence> evidenceFromStream(new SAMLEvidence(s));
            evidenceFromStream->checkValidity();
            cerr << "OK" << endl;
        }

        cerr << "\tOK" << endl;
    }
}

// helper method - create an (arbitrary but valid) assertion
SAMLAssertion *createAssertion(const XMLCh *issuer)
{
    auto_ptr_XMLCh name("Julius Hibbert");
    SAMLNameIdentifier* nameId = new SAMLNameIdentifier(name.get());
    SAMLSubject* sub = new SAMLSubject(nameId);

    auto_ptr_XMLCh val("value");
    const XMLCh* vv[] = { val.get() };

    auto_ptr_XMLCh resource("http://medico.com/record/patient/BartSimpson");
    SAMLAttribute *att = new SAMLAttribute(resource.get(), XML::XML_NS, NULL, 0, ArrayIterator<const XMLCh*>(vv,1));
    SAMLAttribute* attv[] = { att };

    SAMLStatement* s = new SAMLAttributeStatement(sub, ArrayIterator<SAMLAttribute*>(attv,1));
    SAMLStatement* sv[] = { s };

    // we have all the information - create and return the assertion
    return new SAMLAssertion(issuer, NULL, NULL, EMPTY(SAMLCondition*), ArrayIterator<SAMLStatement*>(sv,1));
}

///////////////////////////////////////////////////////////////////////////
// SAMLAuthorizationDecisionQuery
void testSAMLAuthorizationDecisionQuery()
{
    testClass ="SAMLAuthorizationDecisionQuery";
    cerr << endl << testClass << endl;

    {
        // 1 - Query with two actions and evidence
        cerr << "\tQuery with two actions and evidence" << endl;

        auto_ptr_XMLCh name("Julius Hibbert");
        SAMLNameIdentifier* nameId = new SAMLNameIdentifier(name.get());
        SAMLSubject* sub = new SAMLSubject(nameId);

        auto_ptr_XMLCh res("http://medico.com/record/patient/BartSimpson");
        std::vector<SAMLAction*> av;
        auto_ptr_XMLCh readAction("read");
        av.push_back(new SAMLAction(readAction.get()));
        auto_ptr_XMLCh writeAction("write");
        av.push_back(new SAMLAction(writeAction.get()));

        auto_ptr_XMLCh idref("ID666");
        const XMLCh* idrefv[] = { idref.get() };
        SAMLEvidence* evidence = new SAMLEvidence(ArrayIterator<const XMLCh*>(idrefv,1));

        SAMLAuthorizationDecisionQuery* query = new SAMLAuthorizationDecisionQuery(sub, res.get(), av, evidence);

        cerr << "\t\tValid object...";
        query->checkValidity();
        cerr << "OK" << endl;

        // Build from DOMElement
        cerr << "\t\tBuild from DOMElement...";
        SAMLAuthorizationDecisionQuery* queryFromDOM = new SAMLAuthorizationDecisionQuery(static_cast<DOMElement*>(query->toDOM()));
        queryFromDOM->checkValidity();
        delete queryFromDOM;
        cerr << "OK" << endl;

        // getter methods
        cerr << "\t\tGetter methods...";
        if (!XMLString::equals(name.get(), query->getSubject()->getNameIdentifier()->getName())) {
            throw SAMLException("getNameIdentifier() returned incorrect value");
        }
        if (!XMLString::equals(res.get(), query->getResource())) {
            throw SAMLException("getResource() returned incorrect value");
        }
        if (query->getActions().size() != 2) {
            throw SAMLException("getActions() returned incorrect value");
        }
        SAMLAction* action = query->getActions().next();
        if(!XMLString::equals(readAction.get(), action->getData())) {
            throw SAMLException("getData() on first action returned incorrect value");
        }
        const XMLCh* id = query->getEvidence()->getAssertionIDRefs().next();
        if (!XMLString::equals(idref.get(), id)) {
            throw SAMLException("unexpected result for first assertion ID ref");
        }
        cerr << "OK" << endl;

        // clone
        cerr << "\t\tClone...";
        auto_ptr<SAMLAuthorizationDecisionQuery> clonedQuery(static_cast<SAMLAuthorizationDecisionQuery*>(query->clone()));
        delete query;

        clonedQuery->checkValidity();
        if (clonedQuery->getActions().size() != 2) {
            throw SAMLException("getActions() on clone returned incorrect value");
        }

        cerr << "OK" << endl;

        // serialization
        serialize(clonedQuery.get(), gSerializeTo);

        // serialize from input stream
        if(!gSkipDeserializationFromInputStream) {
            cerr << "\t\tSerialize from an input stream...";
            string file = gExamplesDir + testClass + ".xml";
            ifstream s(file.c_str());
            auto_ptr<SAMLAuthorizationDecisionQuery> queryFromStream(new SAMLAuthorizationDecisionQuery(s));
            queryFromStream->checkValidity();
            cerr << "OK" << endl;
        }

        cerr << "\tOK" << endl;
    }
}

///////////////////////////////////////////////////////////////////////////
// SAMLAuthorizationDecisionStatement
void testSAMLAuthorizationDecisionStatement()
{
    testClass = "SAMLAuthorizationDecisionStatement";
    cerr << endl << testClass << endl;

    {
        // 1 - Statement with two actions and evidence
        cerr << "\tStatement with two actions and evidence" << endl;

        auto_ptr_XMLCh name("Julius Hibbert");
        SAMLNameIdentifier* nameId = new SAMLNameIdentifier(name.get());
        SAMLSubject* sub = new SAMLSubject(nameId);

        auto_ptr_XMLCh res("http://medico.com/record/patient/BartSimpson");
        std::vector<SAMLAction*> av;
        auto_ptr_XMLCh readAction("read");
        av.push_back(new SAMLAction(readAction.get()));
        auto_ptr_XMLCh writeAction("write");
        av.push_back(new SAMLAction(writeAction.get()));

        auto_ptr_XMLCh idref("ID666");
        const XMLCh* idrefv[] = { idref.get() };
        SAMLEvidence* evidence = new SAMLEvidence(ArrayIterator<const XMLCh*>(idrefv,1));

        SAMLAuthorizationDecisionStatement* statement = new SAMLAuthorizationDecisionStatement(sub, res.get(), SAMLDecision::Permit, av, evidence);

        cerr << "\t\tValid object...";
        statement->checkValidity();
        cerr << "OK" << endl;

        // Build from DOMElement
        cerr << "\t\tBuild from DOMElement...";
        SAMLAuthorizationDecisionStatement* statementFromDOM =
            new SAMLAuthorizationDecisionStatement(static_cast<DOMElement*>(statement->toDOM()));
        statementFromDOM->checkValidity();
        delete statementFromDOM;
        cerr << "OK" << endl;

        // getter methods
        cerr << "\t\tGetter methods...";
        if (!XMLString::equals(name.get(), statement->getSubject()->getNameIdentifier()->getName())) {
            throw SAMLException("getNameIdentifier() returned incorrect value");
        }
        if (!XMLString::equals(res.get(), statement->getResource())) {
            throw SAMLException("getResource() returned incorrect value");
        }
        if (!XMLString::equals(SAMLDecision::Permit, statement->getDecision())) {
            throw SAMLException("getDecision() returned incorrect value");
        }
        if (statement->getActions().size() != 2) {
            throw SAMLException("getActions() returned incorrect value");
        }
        SAMLAction* action = statement->getActions().next();
        if(!XMLString::equals(readAction.get(), action->getData())) {
            throw SAMLException("getData() on first action returned incorrect value");
        }
        const XMLCh* id = statement->getEvidence()->getAssertionIDRefs().next();
        if (!XMLString::equals(idref.get(), id)) {
            throw SAMLException("unexpected result for first assertion ID ref");
        }
        cerr << "OK" << endl;

        // clone
        cerr << "\t\tClone...";
        auto_ptr<SAMLAuthorizationDecisionStatement> clonedStatement(
            static_cast<SAMLAuthorizationDecisionStatement*>(statement->clone())
            );
        delete statement;

        clonedStatement->checkValidity();
        if (clonedStatement->getActions().size() != 2) {
            throw SAMLException("getActions() on clone returned incorrect value");
        }
        cerr << "OK" << endl;

        // serialization
        serialize(clonedStatement.get(), gSerializeTo);

        // serialize from input stream
        if(!gSkipDeserializationFromInputStream) {
            cerr << "\t\tSerialize from an input stream...";
            string file = gExamplesDir + testClass + ".xml";
            ifstream s(file.c_str());
            auto_ptr<SAMLAuthorizationDecisionStatement> statementFromStream(new SAMLAuthorizationDecisionStatement(s));
            statementFromStream->checkValidity();
            cerr << "OK" << endl;
        }

        cerr << "\tOK" << endl;
    }
}


///////////////////////////////////////////////////////////////////////////
// SAMLAttributeQuery
void testSAMLAttributeQuery()
{
    testClass = "SAMLAttributeQuery";
    cerr << endl << testClass << endl;

    {
        // 1 - Query with two attribute designators
        cerr << "\tQuery with a resource and two attribute designators" << endl;

        auto_ptr_XMLCh name("Julius Hibbert");
        SAMLNameIdentifier* nameId = new SAMLNameIdentifier(name.get());
        SAMLSubject* sub = new SAMLSubject(nameId);

        std::vector<SAMLAttributeDesignator*> adv;
        SAMLAttributeDesignator* att = NULL;
        auto_ptr_XMLCh att1("attributeOne");
        auto_ptr_XMLCh att2("attributeTwo");
        att = new SAMLAttributeDesignator(att1.get(), XML::XPATH2_NS);
        adv.push_back(att);
        att = new SAMLAttributeDesignator(att2.get(), XML::XPATH2_NS);
        adv.push_back(att);

        SAMLAttributeQuery* query = new SAMLAttributeQuery(sub, NULL, adv);

        cerr << "\t\tValid object...";
        query->checkValidity();
        cerr << "OK" << endl;

        // Build from DOMElement
        cerr << "\t\tBuild from DOMElement...";
        SAMLAttributeQuery* queryFromDOM = new SAMLAttributeQuery(static_cast<DOMElement*>(query->toDOM()));
        queryFromDOM->checkValidity();
        delete queryFromDOM;
        cerr << "OK" << endl;

        // getter methods
        cerr << "\t\tGetter methods...";
        if (!XMLString::equals(name.get(), query->getSubject()->getNameIdentifier()->getName())) {
            throw SAMLException("getNameIdentifier() returned incorrect value");
        }
        if (query->getDesignators().size() != 2) {
            throw SAMLException("getDesignators() returned incorrect value");
        }
        SAMLAttributeDesignator* attD = query->getDesignators().next();
        if(!XMLString::equals(att1.get(), attD->getName())) {
            throw SAMLException("getName() on first attribute returned incorrect value");
        }
        cerr << "OK" << endl;

        // clone
        cerr << "\t\tClone...";
        auto_ptr<SAMLAttributeQuery> clonedQuery(static_cast<SAMLAttributeQuery*>(query->clone()));
        delete query;

        clonedQuery->checkValidity();
        if(clonedQuery->getDesignators().size() != 2) {
            throw SAMLException("getDesignators() on clone returned incorrect value");
        }

        cerr << "OK" << endl;

        // serialization
        serialize(clonedQuery.get(), gSerializeTo);

        // serialize from input stream
        if(!gSkipDeserializationFromInputStream) {
            cerr << "\t\tDeserialize from an input stream...";
            string file = gExamplesDir + testClass + ".xml";
            ifstream s(file.c_str());
            auto_ptr<SAMLAttributeQuery> queryFromStream(new SAMLAttributeQuery(s));
            queryFromStream->checkValidity();
            cerr << "OK" << endl;
        }

        cerr << "\tOK" << endl;
    }
}
